@@ -2,7 +2,7 @@ __all__ = [
     "ScalarConstraint",
     "ConstraintError",
     "constraint_function_factory",
-    "ConstraintError",
+    "ConstraintBase",
     "supported_operators",
     "ObjectiveError",
     "ObjectiveEvaluationResults",
@@ -44,6 +44,7 @@ from desdeo_problem.problem import (
     ScalarDataObjective,
     ScalarObjective,
     ConstraintError,
+    ConstraintBase,
     ScalarConstraint,
     constraint_function_factory,
     supported_operators,
